/*=====================================================================================================================

   " ".

      . .

=======================================================================================================================

  :

  04.03.2015 [*] GridModelSet:   .
             [*] GridModelKind:  .
  03.03.2015 [*]    (  ).
  01.03.2015 [+] CalcModelName:   .
  26.02.2015 [+]    (  ).
  19.02.2015 [+] GridModelSet:     .
             [+] FindModel:  .
  07.06.2013 [+] ViewModelPowerName:   .
  08.03.2012 [+] SortModel, MoveModel:    .
             [*] GridModelSet:     .
             [*] GridModelSet:        .
  24.02.2012 [*] .
  19.03.2007 [+] GridModelList:  .
  14.03.2007 [+] GridModelSet:  .
  15.02.2007 [+]  .
  09.10.1997 [!]  .

=======================================================================================================================
  Copyright 19972015 TSK Developer Laboratory, Moscow.                                         Konstantin Tsvetkov
                                                                                                  e-mail: dev@tsklab.ru
                                                                                                  http://www.tsklab.ru
  CVS: $Id: scModel.sql,v 1.8 2015/03/04 06:08:16 tsk Exp $
=====================================================================================================================*/
USE ComputerWare
--
EXECUTE CheckDropProcedure 'GridModelKind'
EXECUTE CheckDropProcedure 'GridModelList'
EXECUTE CheckDropProcedure 'GridModelSet'
EXECUTE CheckDropProcedure 'SortModel'
EXECUTE CheckDropProcedure 'MoveModel'
EXECUTE CheckDropProcedure 'FindModel'
EXECUTE CheckDropProcedure 'CalcModelName'
EXECUTE CheckDropView		   'ViewModelPowerName'
EXECUTE CheckDropTrigger   'ModelInsert'
EXECUTE CheckDropTrigger   'ModelUpdate'
EXECUTE CheckDropTrigger   'ModelDelete'
--
GO
--=====================================================================================================================
CREATE PROCEDURE GridModelKind @KD CHAR(1) = 'H'
AS
  SELECT IdKind AS ID, 
         ISNULL(ShortName, KindName) AS Name,
         KindType
    FROM Kind
    WHERE (KindType = @KD)
    ORDER BY KindOrder
--
GO
--=====================================================================================================================
CREATE PROCEDURE GridModelList @MK INT
AS
  SELECT IdModel, 
         ModlWeight AS ModlWeight, 
         ModlName, 
         ModlParametersValue, 
         ModlNote, 
         ModlKind
    FROM Model
    WHERE (ModlKind = @MK)
--
GO
--=====================================================================================================================
CREATE PROCEDURE GridModelSet @MD INT
AS
  SELECT CompleteSet.IdComponent, 
         ISNULL(ISNULL(ISNULL(CompleteSet.NumberManufact + ISNULL(' [' + CompleteSet.NumberSeller + ']', ''), 
                              CompleteSet.NumberSeller), CmplNumber),'[/]') AS Numb, 
         NameNumber AS DevName,
         Workplace.WrkpName, 
         Department.DprtName, 
         DATEADD(m, CompleteSet.Warranty, Document.DocDate) AS Warranty, 
         ISNULL(DocKind.Name, '') 
           + '  ' + ISNULL(Document.DocNumber, '[/]') 
           + '  ' + ISNULL(CONVERT(VARCHAR(8), Document.DocDate, 4), '[.]') AS Doc, 
         FirmNamePower.NameTrademark AS ClientName,
         CompleteSet.CmplIdDevice AS Device,
         Document.IdDocument
    FROM Document 
      INNER JOIN DocKind ON Document.DocKind = DocKind.ID 
      RIGHT OUTER JOIN CompleteSet INNER JOIN ViewDevicePowerName ON CompleteSet.CmplIdDevice = ViewDevicePowerName.ID
      INNER JOIN Workplace ON ViewDevicePowerName.High = Workplace.IdWorkplace 
      INNER JOIN Department ON Workplace.WrkpIdDepartment = Department.IdDepartment 
                            ON Document.IdDocument = CompleteSet.DocumentId 
      LEFT OUTER JOIN FirmNamePower ON Document.DocClient = FirmNamePower.ID
    WHERE (CompleteSet.CmplIdModel = @MD)
    ORDER BY CompleteSet.CmplNumber
--
GO
--=====================================================================================================================
CREATE PROCEDURE SortModel @MD INT
AS
  DECLARE @KD INT, @SR INT, @NL INT
  SELECT  @KD = ModlKind FROM Model WHERE (IdModel = @MD)
  DECLARE Sort CURSOR FOR SELECT ModlWeight FROM Model WHERE (ModlKind = @KD) ORDER BY ModlWeight
  SET @SR = 1
  OPEN Sort
  FETCH NEXT FROM Sort INTO @NL
  WHILE @@FETCH_STATUS = 0 BEGIN
    UPDATE Model SET ModlWeight = @SR WHERE CURRENT OF Sort
    SET @SR = @SR + 2
    FETCH NEXT FROM Sort INTO @NL
  END
  CLOSE Sort
  DEALLOCATE Sort
--
GO
--=====================================================================================================================
CREATE PROCEDURE MoveModel @MD INT, @DR INT
AS
  -- @DR: -1 - , +1 - 
  SET @DR = @DR * 3
  UPDATE Model SET ModlWeight += @DR WHERE (IdModel = @MD)
  EXECUTE SortModel @MD
--
GO
--=====================================================================================================================
CREATE PROCEDURE FindModel @CM INT
AS
  SELECT CompleteSet.IdComponent, 
         Model.IdModel, 
         Kind.IdKind, 
         Kind.KindType
    FROM CompleteSet 
      INNER JOIN Model ON CompleteSet.CmplIdModel = Model.IdModel 
      INNER JOIN Kind ON Model.ModlKind = Kind.IdKind
    WHERE (CompleteSet.IdComponent = @CM)
--
GO
--=====================================================================================================================
CREATE PROCEDURE CalcModelName @NB VARCHAR(30), @NO BIT, @PR INT, @Name VARCHAR( 60 ) OUT
AS
  DECLARE @PN VARCHAR(30)
  SELECT @PN = ISNULL(Trademark.Name, Client.ClntNameShort) + ' ' 
    FROM Client LEFT OUTER JOIN Trademark ON Client.Trademark = Trademark.ID
    WHERE (Client.IdClient = @PR)
  IF @NO = 1 SET @Name = @NB
    ELSE SET @Name = @PN + @NB
--
GO
--=====================================================================================================================
CREATE VIEW ViewModelPowerName
AS
  SELECT Model.IdModel AS ID, 
         Model.DisplayName AS Name, 
         Model.DisplayNameFull AS NameFull, 
         ISNULL(Kind.ShortName, Kind.KindName) + '. ' + Model.DisplayName AS NameKind
    FROM Model
      INNER JOIN Kind ON Model.ModlKind = Kind.IdKind
--
GO
--=====================================================================================================================
CREATE TRIGGER ModelInsert ON Model FOR INSERT
AS
  SET NOCOUNT ON
  UPDATE Client SET CountModel = (SELECT COUNT(*) FROM Model WHERE ( Model.ModlIdProducer = inserted.ModlIdProducer))
    FROM Client INNER JOIN inserted ON Client.IdClient = inserted.ModlIdProducer
  --
  UPDATE Kind SET UsesModel = (SELECT COUNT(*) FROM Model WHERE ( Model.ModlKind = inserted.ModlKind))
    FROM Kind INNER JOIN inserted ON Kind.IdKind = inserted.ModlKind
--
GO
--=====================================================================================================================
CREATE TRIGGER ModelUpdate ON Model FOR UPDATE
AS
  SET NOCOUNT ON
  UPDATE Client SET CountModel = (SELECT COUNT(*) FROM Model WHERE ( Model.ModlIdProducer = inserted.ModlIdProducer))
    FROM Client INNER JOIN inserted ON Client.IdClient = inserted.ModlIdProducer
  UPDATE Client SET CountModel = (SELECT COUNT(*) FROM Model WHERE (Model.ModlIdProducer = deleted.ModlIdProducer))
    FROM Client INNER JOIN deleted ON Client.IdClient = deleted.ModlIdProducer
  --
  UPDATE Kind SET UsesModel = (SELECT COUNT(*) FROM Model WHERE ( Model.ModlKind = inserted.ModlKind))
    FROM Kind INNER JOIN inserted ON Kind.IdKind = inserted.ModlKind
  UPDATE Kind SET UsesModel = (SELECT COUNT(*) FROM Model WHERE ( Model.ModlKind = deleted.ModlKind))
    FROM Kind INNER JOIN deleted ON Kind.IdKind = deleted.ModlKind
--
GO
--=====================================================================================================================
CREATE TRIGGER ModelDelete ON Model FOR DELETE
AS
  SET NOCOUNT ON
  UPDATE Client SET CountModel = (SELECT COUNT(*) FROM Model WHERE (Model.ModlIdProducer = deleted.ModlIdProducer))
    FROM Client INNER JOIN deleted ON Client.IdClient = deleted.ModlIdProducer
  --
  UPDATE Kind SET UsesModel = (SELECT COUNT(*) FROM Model WHERE ( Model.ModlKind = deleted.ModlKind))
    FROM Kind INNER JOIN deleted ON Kind.IdKind = deleted.ModlKind
--
GO
--=====================================================================================================================
-- 
---- Model:
-- 2205 = Core i3-4370
-- 2206 = GigaByte GTX 750Ti
---- Component:
-- 10391 = Core i5-3570K  2R220384A2496
--
--EXECUTE GridModelKind 'H'
--EXECUTE GridModelSet 857
--EXECUTE GridModelList 96
--EXECUTE SortModel 1340
--EXECUTE MoveModel 1340, 1
--EXECUTE FindModel 10391
DECLARE @N VARCHAR(60)
EXECUTE CalcModelName 'GTX 750Ti', 0, 182, @N OUT
SELECT @N
--SELECT * FROM ViewModelPowerName